/*
 * Brickworks
 *
 * Copyright (C) 2022-2024 Orastron Srl unipersonale
 *
 * Brickworks is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * Brickworks is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Brickworks.  If not, see <http://www.gnu.org/licenses/>.
 *
 * File author: Stefano D'Angelo
 */

#include "impl.h"

#include "common.h"
#include <bw_sr_reduce.h>
#include <bw_bd_reduce.h>

using namespace Brickworks;

class Engine {
public:
	SRReduce<1>	sr;
	BDReduce<1>	bd;
};

extern "C" {

impl impl_new(void) {
	Engine *instance = new Engine();
	instance->bd.setSilenceDc(0);
	return reinterpret_cast<impl>(instance);
}

void impl_free(impl handle) {
	Engine *instance = reinterpret_cast<Engine *>(handle);
	delete instance;
}

void impl_set_sample_rate(impl handle, float sample_rate) {
	Engine *instance = reinterpret_cast<Engine *>(handle);
	instance->sr.setSampleRate(sample_rate);
	instance->bd.setSampleRate(sample_rate);
}

void impl_reset(impl handle) {
	Engine *instance = reinterpret_cast<Engine *>(handle);
	instance->sr.reset();
	instance->bd.reset();
}

void impl_set_parameter(impl handle, size_t index, float value) {
	Engine *instance = reinterpret_cast<Engine *>(handle);
	switch (index) {
	case plugin_parameter_sr_ratio:
		instance->sr.setRatio(0.01f * value);
		break;
	case plugin_parameter_bit_depth:
		instance->bd.setBitDepth((char)value);
		break;
	case plugin_parameter_gate:
		instance->bd.setGateLin(value);
		break;
	}
}

float impl_get_parameter(impl handle, size_t index) {
	(void)handle;
	(void)index;
	return 0.f;
}

void impl_process(impl handle, const float **inputs, float **outputs, size_t n_samples) {
	Engine *instance = reinterpret_cast<Engine *>(handle);
	instance->sr.process(inputs, outputs, n_samples);
	instance->bd.process(outputs, outputs, n_samples);
}

}
